#include <cstdio>
#include <cstring>
#include <stdexcept>
#include <unistd.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <assert.h>
#include <sqlite3.h>

sqlite3* _sqlite3 = nullptr;

bool directory_exists(const char* fs_path)	
{
	struct stat ss;
	if (stat(fs_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFDIR;
}

bool file_exists(const char* file_path)	
{
	struct stat ss;
	if (stat(file_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFREG;
}

std::string get_cesta_db()
{
	std::string db_file = getenv("HOME");
	if (!directory_exists(db_file.c_str()))
		db_file = getpwuid(getuid())->pw_dir;
	if (!directory_exists(db_file.c_str()))
		throw std::runtime_error("Nepodařilo se zjistit domovský adresář.");
	db_file.append("/Dokumenty/cz-trideni.db");
	return db_file;
}

void vytvorit_databazi()
{
	std::string str_file = get_cesta_db();
	if (file_exists(str_file.c_str()))
	{
		if (remove(str_file.c_str()) !=0)
			throw std::runtime_error("Chyba smazání souboru");
	}
	int ires = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	ires = sqlite3_exec(_sqlite3, "CREATE TABLE IF NOT EXISTS polozky (id INTEGER PRIMARY KEY,"
		" obsah TEXT)", nullptr, nullptr, nullptr);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void otevrit_databazi()
{
	std::string str_file = get_cesta_db();
	if (!file_exists(str_file.c_str()))
		throw std::runtime_error("Soubor databáze nebyl nalezen");
	int ires = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void vypsat_data()
{
	assert(_sqlite3);
	sqlite3_stmt* stmt = nullptr;
	char sz_sql[255];
	strcpy(sz_sql, "SELECT obsah FROM polozky ORDER BY obsah COLLATE cestina");
	int ires = sqlite3_prepare_v2(_sqlite3, sz_sql,
		static_cast<int>(strlen(sz_sql)), &stmt, nullptr);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	int istep;
	istep = sqlite3_step(stmt);
	while (SQLITE_ROW == istep)
	{
		printf("%s\n", (char*)sqlite3_column_text(stmt, 0));
		istep = sqlite3_step(stmt);
		if (SQLITE_ERROR == istep)
			throw std::runtime_error(sqlite3_errstr(istep));
	}
	ires = sqlite3_reset(stmt);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	ires = sqlite3_finalize(stmt);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void naplnit_databazi()
{
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('CC')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('cc')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('čč')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('ČČ')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('XX')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('xx')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('áá')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('aa')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('ch')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('CH')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('GG')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('jj')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('ii')",
		nullptr, nullptr, nullptr);
	sqlite3_exec(_sqlite3, "INSERT INTO polozky (obsah) VALUES ('íí')",
		nullptr, nullptr, nullptr);
}

int porovnat_cesky(void*,int delka1, const void* text1, int delka2, const void* text2)
{
	return strcoll((const char*)text1, (const char*)text2);
}

int main(int argc, char** argv)
{
	setlocale(LC_COLLATE, "cs_CZ.utf8");
	try
	{
		if (file_exists(get_cesta_db().c_str()))
		{
			otevrit_databazi();
			sqlite3_create_collation(_sqlite3, "cestina", SQLITE_UTF8, nullptr, porovnat_cesky);
			vypsat_data();
		}
		else
		{
			vytvorit_databazi();
			naplnit_databazi();
		}
		sqlite3_close(_sqlite3);
		printf("\nok\n");
		return EXIT_SUCCESS;
	}
	catch(const std::exception& e)
	{
		printf("Došlo k chybě: %s\n", e.what());
		if (_sqlite3)
			sqlite3_close(_sqlite3);
		return EXIT_FAILURE;
	}
}

